@@ -7,6 +7,53 @@ using GLib;
 using System.Runtime.InteropServices;
 
 namespace FSpot {
+	public class XScreenSaverSlide : Gtk.Window {
+		public const string ScreenSaverEnviroment = "XSCREENSAVER_WINDOW";
+
+		public XScreenSaverSlide () : base ("")
+		{
+		}
+	       
+		protected override void OnRealized ()
+		{
+			string env = Environment.GetEnvironmentVariable (ScreenSaverEnviroment);
+			
+			if (env != null) {
+				try {
+					env = env.ToLower ();
+					
+					if (env.StartsWith ("0x"))
+						env = env.Substring (2);
+
+					uint xid = UInt32.Parse (env, System.Globalization.NumberStyles.HexNumber);
+					
+					GdkWindow = Gdk.Window.ForeignNew (xid);
+					GdkWindow.Events = EventMask.ExposureMask | EventMask.StructureMask | EventMask.EnterNotifyMask 
+						| EventMask.LeaveNotifyMask | EventMask.FocusChangeMask;
+					
+					Style.SetBackground (GdkWindow, Gtk.StateType.Normal);
+					GdkWindow.SetDecorations ((Gdk.WMDecoration) 0);
+					GdkWindow.UserData = this.Handle;
+					SetFlag (WidgetFlags.Realized);
+					SizeRequest ();
+					Gdk.Rectangle geom;
+					int depth;
+					GdkWindow.GetGeometry (out geom.X, out geom.Y, out geom.Width, out geom.Height, out depth);
+					SizeAllocate (new Gdk.Rectangle (geom.X, geom.Y, geom.Width, geom.Height));
+					Resize (geom.Width, geom.Height);
+					return;
+				} catch (System.Exception e) {
+					System.Console.WriteLine (e);
+				}
+			} else {
+				System.Console.WriteLine ("{0} not set, falling back to window", ScreenSaverEnviroment);
+			}
+
+			SetSizeRequest (640, 480);
+			base.OnRealized ();
+		}
+	}
+
 	public class FullSlide : Gtk.Window {
 		private SlideView slideview;
 		private Gdk.Pixbuf screenshot;
@@ -100,11 +147,6 @@ namespace FSpot {
 		private void HandleDestroyed (object sender, System.EventArgs args)
 		{
 			hide.Stop ();
-
-			this.busy.Unref ();
-			if (this.none != null)
-				this.none.Unref ();
-
 		}
 
 		private void HandleSlideViewButtonPressEvent (object sender, ButtonPressEventArgs args)
@@ -131,8 +173,9 @@ namespace FSpot {
 		uint flip_timer = 0;
 		uint transition_timer = 0;
 		
+		uint fail_count = 0;
 		bool animate = true;
-		uint animate_max = 160;
+		uint animate_max = 200;
 		
 		bool black = false;
 		uint flip_interval = 2000;
@@ -145,16 +188,15 @@ namespace FSpot {
 		}
 		
 		public bool Animate {
-			get {
-				return animate;
-			}
-			set {
-				animate = value;
-			}
+			get { return animate; }
+			set { animate = value; }
 		}
 		
 		public void Play () 
 		{
+			if (photos.Length < 1)
+				return;
+
 			StopTweenIdle ();
 			if (current_idx >= 0) {
 				Pixbuf frame = GetScaled (photos[current_idx]);
@@ -196,7 +238,7 @@ namespace FSpot {
 			StopTweenIdle ();
 			
 			if (current_idx != next_idx && next != null)
-				this.FromPixbuf = next;
+				this.Pixbuf = next;
 			
 			current_idx = next_idx;
 			
@@ -213,15 +255,17 @@ namespace FSpot {
 						next.Dispose ();
 					
 					next = GetScaled (photos [idx]);
-					
+					if (next == null)
+						next = GetScaled (PixbufUtils.ShallowCopy (PixbufUtils.ErrorPixbuf));
 					
 					next_idx = idx;
 					StartTweenIdle ();
 					
-
 					return true;
 				} else {
-					next.Dispose ();
+					if (next != null)
+						next.Dispose ();
+
 					next = GetScaled (photos [0]);
 					next_idx = 0;
 					StartTweenIdle ();
@@ -282,6 +326,9 @@ namespace FSpot {
 		private Pixbuf GetScaled (IBrowsableItem photo)
 		{
 			Pixbuf orig = FSpot.PhotoLoader.LoadAtMaxSize (photo, Allocation.Width, Allocation.Height);
+			if (orig == null)
+				return null;
+
 			Pixbuf result = GetScaled (orig);
 			if (orig != result)
 				orig.Dispose ();
@@ -301,23 +348,31 @@ namespace FSpot {
 		
 		private bool HandleTransitionTimer ()
 		{			
+			System.DateTime start_time = System.DateTime.Now;
 			transition_timer = 0;
 			if (current_tween--  > 0) {
 				StartTransitionTimer ();
-				System.DateTime start_time = System.DateTime.Now;
-				this.FromPixbuf = tweens[current_tween];
+				this.Pixbuf = tweens[current_tween];
 				GdkWindow.ProcessUpdates (false);
 				System.TimeSpan span = System.DateTime.Now  - start_time;
 				
-				if (animate && span.TotalMilliseconds > animate_max) {
-					animate = false;
-					System.Console.WriteLine ("Disabling slide animation due to excessive frame interval {0}ms", 
-								  span.TotalMilliseconds);
-					current_tween = 0;
-				}
+				if (Animate) { 
+					if (span.TotalMilliseconds > animate_max) {
+						fail_count++;
+						
+						if (fail_count > 3) {
+							Animate = false;
+							System.Console.WriteLine ("Disabling slide animation due to 3 consecutive excessive frame intervals {0}ms", 
+										  span.TotalMilliseconds);
+							current_tween = 0;
+						}
+					} else {
+						fail_count = 0;
+					}
+				} 
 			} else {
 				ShowNext ();
-				
+
 				PreloadNextImage (current_idx + 1);
 				StartFlipTimer ();
 			}
@@ -329,7 +384,12 @@ namespace FSpot {
 		private bool HandleTweenIdle ()
 		{
 			using (Pixbuf prev = this.Pixbuf) {	
-				if (!animate) {
+				if (!Animate) {
+					ClearTweens ();
+					return false;
+				}
+				
+				if (photos.Length < 2) { // Only one photo. Nothing to do
 					ClearTweens ();
 					return false;
 				}
@@ -384,9 +444,9 @@ namespace FSpot {
 		
 		private void StopTranstionTimer ()
 		{
-			if (transition_timer != 0) {
+			if (transition_timer != 0)
 				GLib.Source.Remove (transition_timer);
-			}
+
 			transition_timer = 0;
 		}
 		
@@ -399,33 +459,34 @@ namespace FSpot {
 		
 		private void StopFlipTimer ()
 		{	
-			if (flip_timer != 0) {
+			if (flip_timer != 0)
 				GLib.Source.Remove (flip_timer);
-			}
+
 			flip_timer = 0;
 		}
 
 		
 		private void HandleSizeAllocate (object sender, SizeAllocatedArgs args)
 		{	
-			if (Pixbuf == null)
+			Pixbuf current = this.Pixbuf;
+
+			if (current == null)
 				return;
-			
+
 			//
 			// The size has changed so we need to reload the images.
 			//
-			Pixbuf current = this.Pixbuf;
 			if (current.Width != Allocation.Width || current.Height != Allocation.Height) {
 				bool playing = (flip_timer != 0 || transition_timer != 0);
 				
 				if (current_idx < 0) {
-					Gdk.Pixbuf old = this.Pixbuf;
-					this.Pixbuf = GetScaled (this.Pixbuf);
-					if (old != this.Pixbuf)
-						old.Dispose ();
+					using (Gdk.Pixbuf old = this.Pixbuf) {
+						this.Pixbuf = GetScaled (old);
+						current.Dispose ();
+					}
 				} else {
 					using (Pixbuf frame =  GetScaled (photos[current_idx])) {
-						this.FromPixbuf =  frame;
+						this.Pixbuf =  frame;
 						current.Dispose ();
 					}
 				}
@@ -443,9 +504,9 @@ namespace FSpot {
 
 		private void ClearTweens () {
 			for (int i = 0; i < tweens.Length; i++) {
-			if (tweens[i] != null) 
-				tweens[i].Dispose ();
-			tweens[i] = null;
+				if (tweens[i] != null) 
+					tweens[i].Dispose ();
+				tweens[i] = null;
 			}
 		}
 		
@@ -460,7 +521,7 @@ namespace FSpot {
 			this.photos = photos;
 
 			if (background != null) {
-				this.FromPixbuf = background;
+				this.Pixbuf = background;
 				background.Dispose ();
 				
 				current_idx = -1;
